/* genetx7.c
 * convert DX7/TX7 files from genedit to sysex-ready dump files
 * (C) JJR 26121993
 * Last update: 26121993
 */

#include <stdio.h>
#include <filename.h>	/* JJRlib */
#include <stat.h>
#include <unistd.h>
#include <boolean.h>		/* JJRlib */
#include <string.h>
#include <stdlib.h>

#define VERSION	"1.0a"

char *OutfileName(const char *Infile, const char *Ext, const char *OutputDir);
void Usage(void);
void Handle4340(FILE *In, const char *OutputFile, int Size);
void Handle8436(FILE *In, const char *OutputFile, int Size);

static char Ext[4] = "txx";
static BOOLEAN StripOnly = FALSE;
static BOOLEAN Overwrite = FALSE;
static BOOLEAN UseSameName = FALSE;
static DX7 = FALSE;

void main(int argc, char *argv[])
{
	FILE *In;
	char OutputFile[256];
	char OutputDir[256] = "";
	struct stat FileStat;

	if (argc < 2) {
		Usage();
		exit(1);
	}

	for (argc--, argv++; argc > 0; argc--, argv++) {
		if (**argv == '-') {
			switch ((*argv)[1]) {
			case 's':					/* strip config only */
				StripOnly = TRUE;
				break;
			case 'e':					/* new extension */
				argv++, argc--;
				strncpy(Ext, *argv, 3);
				break;
			case 'd':
				argv++, argc--;
				strcpy(OutputDir, *argv);
				break;
			case 'O':
				UseSameName = TRUE;
				/* fall through */
			case 'o':
				Overwrite = TRUE;
				break;
			case 'D':
				DX7 = TRUE;
				break;
			default:
				printf("Unknown switch: -%c\n", (*argv)[1]);
			}
			continue;
		}

		if (NULL == (In = fopen(*argv, "rb"))) {
			printf("Cannot open input file %s.\n", *argv);
			continue;
		}

		if (UseSameName)
			strcpy(OutputFile, *argv);
		else
			strcpy (OutputFile, OutfileName(*argv, Ext, OutputDir));
		printf("  %s -> %s\n", *argv, OutputFile);

		if (!Overwrite && 0 == access(OutputFile, F_OK)) {
			printf("Output file %s exists already. Will not overwrite.\n", OutputFile);
			fclose(In);
			continue;
		}

		stat(*argv, &FileStat);
		switch(FileStat.st_size) {
		case 4340:
		case 4342:
			Handle4340(In, OutputFile, FileStat.st_size);
			break;
		case 8436:
		case 16752:
		case 16754:
			Handle8436(In, OutputFile, FileStat.st_size);
			break;
		default:
			printf("Sorry - can't handle file of size %d yet.\n",
					FileStat.st_size);
		}
		fclose(In);
#if 0
		printf("%s done\n", *argv);
		fflush(stdout);
#endif
	}
}

/***************************************************************************
* OutfileName
* ACTIONS: Generates output file name based on the input file name
* PARAMETERS: char *Infile
*             char *Ext
*             char *OutputDir
* RETURNS: ptr to output file name in a local buffer
* WARNINGS:
***************************************************************************/
char *OutfileName(const char *Infile, const char *Ext, const char *OutputDir)
{
	char Buffer[256];

	if (0 != strcmp(OutputDir, "")) {	/* different output dir */
		char Buffer2[256], Buffer3[15];
		AddSlash(strcpy(Buffer2, OutputDir));
		strcat(Buffer2, Basename(Infile, Buffer3));
		ReplaceLastExtension(Buffer2, Ext, Buffer);
	}
	else
		ReplaceLastExtension(Infile, Ext, Buffer);
	return Buffer;
}

/***************************************************************************
* Usage
* ACTIONS: Tell user how this program should be used
* PARAMETERS: void
* RETURNS: none
* WARNINGS: none
***************************************************************************/
void Usage(void)
{
	printf("genetx7 " VERSION " (C) JJR 1993\n"
			 "Usage:\n"
			 "  genetx7 [-s][-e ext][-o] files...\n");
	printf("Converts Genedit formatted DX7/TX7 files to loadable bank data dumps\n");
	printf("Switches:\n"
			 "  -s       only strip Genedit configuration\n"
			 "  -e ext   create new file with specified extension .ext (default is .txx)\n"
			 "  -d dir   write output file to specified directory dir\n"
			 "  -o       overwrite files (default: do not overwrite\n"
			 "  -O       use input file name as output file name (implies overwrite)\n"
			 "  -D       create DX7 dump files (leave function data out) [overrides -s]\n");
}

/***************************************************************************
* Handle4340
* ACTIONS: Handle files of size 4340 or 4342 bytes: (DX7 without config)
*          normal: skip 240 bytes / write 4096 bytes / skip the rest
*          StripOnly: error - already stripped
* PARAMETERS: FILE *In
*             char *OutputFile
*             int Size
* RETURNS:
* WARNINGS:
***************************************************************************/
void Handle4340(FILE *In, const char *OutputFile, int Size)
{
	char *Buffer;
	FILE *Out;

	if (StripOnly) {
		printf("-- already stripped\n");
		return;
	}

	/* create buffer */
	if (NULL == (Buffer = (char *) malloc(Size * sizeof(char)))) {
		printf("Cannot malloc space for input buffer... exiting\n");
		exit(1);
	}

	/* read input */
	fread(Buffer, sizeof(char), Size, In);

	if (NULL == (Out = fopen(OutputFile, "wb")))
		printf("Cannot open output file %s.\n", OutputFile);
	else {	/* write to output */
		fwrite(Buffer+240, sizeof(char), 4096, Out);
		fclose(Out);
	}

	free(Buffer);
}

/***************************************************************************
* Handle8436
* ACTIONS: Handle files of size 8436, 16752 or 16754 bytes (TX7 data and
*          parameters without or with config)
*          DX7: skip 240 bytes / write 4096 bytes / skip the rest
*          normal: skip 240 bytes / write 4096+4096 bytes / skip the rest
*          StripOnly:
*            8436: error - already stripped
*            16752, 16754: write 8432 bytes / skip... / write last 4 bytes
* PARAMETERS: FILE *In
*             char *OutputFile
*             int Size
* RETURNS:
* WARNINGS:
***************************************************************************/
void Handle8436(FILE *In, const char *OutputFile, int Size)
{
	char *Buffer;
	FILE *Out;

	if (Size == 8436 && StripOnly) {
		printf("-- already stripped\n");
		return;
	}

	/* create buffer */
	if (NULL == (Buffer = (char *) malloc(Size * sizeof(char)))) {
		printf("Cannot malloc space for input buffer... exiting\n");
		exit(1);
	}

	/* read input */
	fread(Buffer, sizeof(char), Size, In);

	if (NULL == (Out = fopen(OutputFile, "wb")))
		printf("Cannot open output file %s.\n", OutputFile);
	else {	/* write to output */
		if (DX7)
			fwrite(Buffer+240, sizeof(char), 4096, Out);
		else
			switch(StripOnly) {
			case FALSE:
				fwrite(Buffer+240, sizeof(char), 2*4096, Out);
				break;
			case TRUE:
				fwrite(Buffer, sizeof(char), 8432, Out);
				fwrite(Buffer+Size-4, sizeof(char), 4, Out);
				break;
			}
		fclose(Out);
	}
	free(Buffer);
}
